//
//  FDynamicTree.m
//  FunInputToy2.0
//
//  Created by Feng Huajun on 07-12-28.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "FDynamicTree.h"


@implementation FDynamicTree

- (id) initFromFile:(const char*) filename withType:(int) t
{
	self = [super init];
	if (self != nil) {
		type = t;
		NSFileManager* fs = [NSFileManager defaultManager];
		NSString* path = [NSString stringWithUTF8String:filename];
		if(![fs fileExistsAtPath:path]){
			db = [[FSQLiteDB alloc] initWithPath:path];
			[db execute:@"CREATE TABLE words (word_id INTEGER PRIMARY KEY AUTOINCREMENT, code TEXT, string TEXT, hit INTEGER, freq INTEGER, is_new INTEGER, from_net INTEGER, sent INTEGER);"];
			[db execute:@"CREATE INDEX words_index ON words (code,string,hit,freq,is_new,from_net,sent);"];
		}else{
			db = [[FSQLiteDB alloc] initWithPath:path];
		}
		updatedWords = [[NSMutableArray alloc] init];
		[self loadWordsFromDB];
		[[NSNotificationCenter defaultCenter] addObserver:self
				selector:@selector(updateDBNow:)
				name:@"UpdateWords" object:nil];		
	}
	return self;
}

-(void) loadWordsFromDB
{
	if(!root) [root release];
	root = [[FDynamicTreeNode alloc] initWithCodeIndex:0];
	NSArray* res = [db query:@"select * from words"];
	NSEnumerator* e = [res objectEnumerator];
	NSDictionary* row;
	while(row = [e nextObject]){
		FWord* w = [FWordFactory genNewWord:type];
		[w setWordId:[[row objectForKey:@"word_id"] intValue]];
		[w setCode:[row objectForKey:@"code"]];
		[w setString:[row objectForKey:@"string"]];
		[w setHit:[[row objectForKey:@"hit"] intValue]];
		[w setFreq:[[row objectForKey:@"freq"] intValue]];
		[w setNewFlag:[[row objectForKey:@"is_new"] intValue]];
		[w setSentFlag:[[row objectForKey:@"sent"] intValue]];
		[self addWordWithoutDB:w];
		//[w release];
	}
}

-(void) updateWord:(FWord*) w
{
	NSString* sql;
	//NSMutableString* code = [NSMutableString stringWithString:[w code]];
	//[code replaceOccurrencesOfString:@"'" withString:@"''" options: range:NSMakeRange(0, [code length])];
	if([w wordId]){
		sql = [NSString stringWithFormat:@"update words set code=\"%@\",string='%@',hit=%d, freq=%d,is_new=%d, sent=%d where word_id=%d",
																[w code], [w string],[w hit], [w freq],[w isNew], [w isSent], [w wordId]];
		[db execute:sql];
    }else{
		sql = [NSString stringWithFormat:@"insert into words (code,string,hit, freq,is_new, sent, from_net)values(\"%@\",'%@',%d, %d,%d, 0, 0)",
																[w code], [w string],[w hit],[w freq], [w isNew]];
		[db execute:sql];
		[w setWordId:[db last_insert_id]];
	}
}

-(void) deleteWord:(FWord*) w
{
	NSString* sql;
	sql = [NSString stringWithFormat:@"delete from words where word_id=%d",[w wordId]];
	[db execute:sql];
}

-(FWord*) addWordWithoutDB:(FWord*) w
{
	UInt32 i;
	UInt32 cc[100];
	UInt32 len = [w genCodeArray:cc];
	FDynamicTreeNode* node = root;
	FDynamicTreeNode* tmpNode;
	for(i=0;i<len;i++){
		tmpNode = node;
		node = [tmpNode childNodeForCodeIndex:cc[i]];
		if(node==nil){
			node = [[FDynamicTreeNode alloc] initWithCodeIndex:cc[i]];
			[tmpNode addChildNode:node];
			[node release];
		}
	}
	return [node addWord:w];	
}

-(void) addWord:(FWord*) w
{
	FWord* ww = [FWordFactory genNewWord:type];
	[ww setCode:[w code]];
	[ww setString:[w string]];
	[ww setHit:1];
	[ww setNewFlag:[w isNew]];
	FWord* rw = [self addWordWithoutDB:ww];
	//[self performSelector:<#(SEL)aSelector#> withObject:<#(id)anArgument#> afterDelay:<#(NSTimeInterval)delay#> inModes:<#(NSArray *)modes#>
	//[self updateWord:rw];
	[updatedWords addObject:rw];
	
	#ifdef LEOPARD
	
	[self performSelector:@selector(updateDBNow:) withObject:nil afterDelay:1.0];
	
	#else
	
		#ifndef IPHONE
	
		[self updateDBNow:nil];
	
		#endif
	
	#endif
	
}


-(void) addWordToMB:(FWord*) w
{
	//FWord* rw = [self addWordWithoutDB:ww];
	//[self performSelector:<#(SEL)aSelector#> withObject:<#(id)anArgument#> afterDelay:<#(NSTimeInterval)delay#> inModes:<#(NSArray *)modes#>
	//[self updateWord:rw];
	[updatedWords addObject:w];	
}

-(void) removeWord:(FWord*) w
{
	UInt32 i;
	UInt32 cc[100];
	UInt32 len = [w genCodeArray:cc];
	FDynamicTreeNode* node = root;
	FDynamicTreeNode* tmpNode;
	for(i=0;i<len;i++){
		tmpNode = node;
		node = [tmpNode childNodeForCodeIndex:cc[i]];
		if(node==nil) return;
	}
	[self deleteWord:w];
	[node removeWord:w];
}

-(NSArray*) wordsByCodeIndexs:(UInt32 *) cc withLength:(UInt32) len returnLength:(UInt32*) oLen
{
	[NSObject cancelPreviousPerformRequestsWithTarget:self];
	UInt32 i;
	FDynamicTreeNode* node = root;
	FDynamicTreeNode* tmpNode;
	for(i=0;i<len;i++){
		tmpNode = [node childNodeForCodeIndex:cc[i]];
		if(tmpNode==nil){
			break;
		}
		node = tmpNode;
	}
	*oLen = i;
	return [node words];
}



-(NSArray*) wordsByMutiCodeIndexs:(UInt32*) xcc returnIndex:(UInt32*) oIndex
{

	#ifndef IPHONE
	[NSObject cancelPreviousPerformRequestsWithTarget:self];
	#endif
	
	NSMutableArray* res = [NSMutableArray array];
	
	NSAutoreleasePool * tmp_pool = [[NSAutoreleasePool alloc] init];
	
	NSMutableArray* r = [NSMutableArray array];
	FDynamicTreeNode* emptyNode = [[[FDynamicTreeNode alloc] initWithCodeIndex:0] autorelease];
	
	FDynamicTreeNode* node;
	FDynamicTreeNode* tmpNode;
	
	UInt32 i = 0,ri=0;

	int ir[100], ir_count =-1, oi;

	[r addObject:root];
	[r addObject:emptyNode];
	
	//UInt32 len = 0;
	
	while(xcc[i]!=0){
		
		ir_count++;
		ir[ir_count] = i;
		
		node = [r objectAtIndex:ri];
		UInt32 sum = 0;
		do{
		  UInt32 j = i;
		  while(xcc[j] != 0){
			tmpNode = [node childNodeForCodeIndex:xcc[j]];
			if(tmpNode != nil)
			{
				//flag = YES;
				[r addObject:tmpNode];
				sum++;
			}
			j++;
		 }
		 ri++;
		 node = [r objectAtIndex:ri];
		}while(node!=emptyNode);
		
		if(sum == 0) {
			oi = ir[ir_count];
			ir_count--;
			break;
		}
		
		
		[r addObject:emptyNode];
		
		while(xcc[i]!=0)i++;
		i++;		
		//if(flag) len ++;
		ri++;

	}
	oi = i;
	//printf("oi1=%d\n",oi);
	//for(i=0;i<wi;i++) printf("r[%d]=%d\n",i,r[i]);
	//*oLen = len;
	
	//NSLog(@"len=%d", len);
	
	UInt32 wi = [r count]-1;
	do {
		while([r objectAtIndex:wi]==emptyNode) wi--;
		while(wi>1 && (tmpNode = [r objectAtIndex:wi])!=emptyNode){
			if([tmpNode words]){
				[res addObjectsFromArray:[tmpNode words]];
			}
			wi--;
		}
		if([res count]) break;
		if(wi>1){
			oi = ir[ir_count];
			ir_count--;
		}
	}while(wi>1);
	
	*oIndex = oi;
	
	//printf("oi=%d\n",oi);
	
	[tmp_pool release];
	
	return res;
}

-(NSArray*) userWords
{
	[self updateDBNow:nil];
	NSMutableArray* ws = [NSMutableArray array];
	NSArray* res = [db query:@"select * from words where is_new=1"];
	NSEnumerator* e = [res objectEnumerator];
	NSDictionary* row;
	while(row = [e nextObject]){
		FWord* w = [FWordFactory genNewWord:type];
		[w setWordId:[[row objectForKey:@"word_id"] intValue]];
		[w setCode:[row objectForKey:@"code"]];
		[w setString:[row objectForKey:@"string"]];
		[w setHit:[[row objectForKey:@"hit"] intValue]];
		[w setNewFlag:[[row objectForKey:@"is_new"] intValue]];
		[w setSentFlag:[[row objectForKey:@"sent"] intValue]];
		[ws addObject:w];
	}
	return ws;
}

-(NSArray*) userWordsForPost
{
	[self updateDBNow:nil];
	NSMutableArray* ws = [NSMutableArray array];
	NSArray* res = [db query:@"select * from words where is_new=1 and sent=0"];
	NSEnumerator* e = [res objectEnumerator];
	NSDictionary* row;
	while(row = [e nextObject]){
		FWord* w = [FWordFactory genNewWord:type];
		[w setWordId:[[row objectForKey:@"word_id"] intValue]];
		[w setCode:[row objectForKey:@"code"]];
		[w setString:[row objectForKey:@"string"]];
		[w setHit:[[row objectForKey:@"hit"] intValue]];
		[w setNewFlag:[[row objectForKey:@"is_new"] intValue]];
		[w setSentFlag:[[row objectForKey:@"sent"] intValue]];
		if([[w string] length] > 4) continue;
		[ws addObject:w];
	}
	return ws;
}

-(void) userWordsPostDone
{
	[db query:@"update words set sent=1 where sent=0"];
}

-(void) updateDBNow:(NSObject*) sender
{
	//NSLog(@"update db now %@",updatedWords);
	NSEnumerator* e = [updatedWords objectEnumerator];
	FWord* w;
	[db beginTrans];
	while(w = [e nextObject]){
		[self updateWord:w];
	}
	[db endTrans];
	[updatedWords removeAllObjects];
}

- (void) dealloc
{
	[self updateDBNow:nil];
	[updatedWords release];
	[db release];
	[root release];
	[super dealloc];
}


@end
