//
//  FInputEngine.m
//  FunInputToy2.0
//
//  Created by Feng Huajun on 08-1-23.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "FInputEngine.h"


@implementation FInputEngine

-(id) initWithDataPath:(NSString*) dataPath withUserPath:(NSString*) userDataPath withSetting:(NSDictionary*)setting
{
	self = [super init];
	if (self != nil) {
		NSString* userPath = userDataPath;
		if([userPath characterAtIndex:0]=='~') {
			userPath = [NSHomeDirectory() stringByAppendingString:[userPath substringFromIndex:1]];
		}
		pyDict = [[FPYDictionary alloc] initWithIndexFile:[dataPath stringByAppendingString:@"/py.idx"]
									listFile:[dataPath stringByAppendingString:@"/py.dat"]
									dbFile:[userPath stringByAppendingString:@"/py.sqlite"]
									spSchemeFile:[dataPath stringByAppendingString:@"/sp.conf"]];
		wbxDict = [[FWBXDictionary alloc] initWithIndexFile:[dataPath stringByAppendingString:@"/wbx.idx"]
													  listFile:[dataPath stringByAppendingString:@"/wbx.dat"] 
													  dbFile:[userPath stringByAppendingString:@"/wbx.sqlite"]];
													  
		bhDict =  [[FBHDictionary alloc] initWithIndexFile:[dataPath stringByAppendingString:@"/bh.idx"]
													  listFile:[dataPath stringByAppendingString:@"/bh.dat"] 
													  dbFile:[userPath stringByAppendingString:@"/bh.sqlite"]];
													  
		relatedDict = [[FRelatedWordList alloc] initWithIndexFile:[dataPath stringByAppendingString:@"/related.idx"] 
											listFile:[dataPath stringByAppendingString:@"/related.dat"]];	

		pyNumDict = [[FPYNumDictionary alloc] initWithPYDict:pyDict];
		
		currentDict = pyDict;
	
		non_dhsplz_set = [[[NSCharacterSet characterSetWithCharactersInString:@"dhspnz"] invertedSet] retain];
		non_char_set = [[[NSCharacterSet characterSetWithCharactersInString:@"!@#$%?"] invertedSet] retain];
	
		
		currentInputMode = FPYInputMode;
		wbxPYMix = NO;
		tmpPY = NO;
		tmpBH = NO;
		[self reloadSetting:setting];
	}
	
	return self;
}

-(FInputMode) currentInputMode
{
	return currentInputMode;
}

-(void) reloadSetting:(NSDictionary*)setting
{
	NSString* inputMode = [setting objectForKey:@"InputMode"];
	if([inputMode isEqualToString:@"PY"]) [self setInputMode:FPYInputMode];
	else if([inputMode isEqualToString:@"WBX"]) [self setInputMode:FWBXInputMode];
	else if([inputMode isEqualToString:@"BH"]) [self setInputMode:FBHInputMode];
	else if([inputMode isEqualToString:@"HR"]) [self setInputMode:FBHInputMode];
	
	NSString* pyMode = [setting objectForKey:@"PYMode"];
	if([pyMode isEqualToString:@"SP"]) [FPYUtils setPYMode:FSPMode];
	else if([pyMode isEqualToString:@"QPSP"]) [FPYUtils setPYMode:FQPSPMode];
	else if([pyMode isEqualToString:@"NUM"]) 
	{
		[FPYUtils setPYMode:FQPMode];
		if([inputMode isEqualToString:@"PY"])  [self setInputMode:FPYNumInputMode];
	}
	else [FPYUtils setPYMode:FQPMode];

	NSString* spScheme = [setting objectForKey:@"SPScheme"];
	if(spScheme) [FPYUtils selectSPScheme:spScheme];

	NSString* wbxMode = [setting objectForKey:@"WBXMode"];
	wbxPYMix = [wbxMode isEqualToString:@"Mix"];

	NSDictionary* mhPYSetting = [setting objectForKey:@"MHPYSetting"];
	[FPYUtils setMHByDictionary:mhPYSetting];

	if([pyMode isEqualToString:@"NUM"])	
		[pyNumDict reloadNumPYDict];
}

-(void) setInputMode:(FInputMode) m
{
	currentInputMode = m;
	if(currentInputMode == FPYInputMode) currentDict = pyDict;
	else if(currentInputMode == FWBXInputMode) currentDict = wbxDict;
	else if(currentInputMode == FPYNumInputMode) currentDict = pyNumDict;
	else currentDict = bhDict;
}

-(NSString*) fixString:(NSString*)s
{
	tmpBH = NO;
	//NSLog(@"s=%@",s);
	if(currentDict==pyNumDict){
		NSRange r = [s rangeOfCharacterFromSet:non_char_set];
		//NSLog(@"%@,%d,%d",s,r.location,NSNotFound);
		if(r.location == NSNotFound){
			tmpBH = YES;
			return s;
		}
	}
	
	if(currentDict==pyDict && [FPYUtils currentPYMode] == FQPMode && [s hasPrefix:@"u"]){
		tmpBH = YES;
		return s;
	}
	
	if([s length] >= 5){
		NSRange r = [s rangeOfCharacterFromSet:non_dhsplz_set];
		if(r.location == NSNotFound){
			tmpBH = YES;
			return s;
		}
	}

	
	if(currentInputMode==FWBXInputMode && wbxPYMix && [s length] > 4){
		return [pyDict fixInputString:s];
	}
	return [currentDict fixInputString:s];
}

-(NSArray*) wordsSearchByFixedString:(NSString*)s
{
	tmpPY = NO;
	
	if(tmpBH){
		if([s hasPrefix:@"u"])
			return [bhDict searchWordsByFixedString:[s substringFromIndex:1]];
		else
			return [bhDict searchWordsByFixedString:s];
	}
	
	
	NSArray* res = [currentDict searchWordsByFixedString:s];
	if(currentInputMode==FWBXInputMode && wbxPYMix){
		if([res count]==0||[FPYUtils isPY:s]){
			if([res count]==0) tmpPY = YES;
			NSString* fix = [pyDict fixInputString:s];
			NSArray* res2 = [pyDict searchWordsByFixedString:fix];
			res = [res arrayByAddingObjectsFromArray:res2]; 
		}
	}
	
	NSMutableArray* resx = [NSMutableArray array];
	NSMutableSet* rset = [NSMutableSet set];
	
	NSEnumerator* e = [res objectEnumerator];
	FWord* w;
	while(w = [e nextObject]){
		if(![rset containsObject:[w string]]){
			[rset addObject:[w string]];
			[resx addObject:w];
		}
	}
	
	return resx;
}

-(NSArray*) relatedWordsByString:(NSString*) s
{
	if(!s || [s length]==0) return nil;
	NSString* ss = s;
	if([s length] > 0) ss = [s substringFromIndex:[s length]-1];
	NSArray* res = [relatedDict relatedWordsByString:ss];
	
	NSMutableArray* resx = [NSMutableArray array];
	NSMutableSet* rset = [NSMutableSet set];
	
	NSEnumerator* e = [res objectEnumerator];
	FWord* w;
	while(w = [e nextObject]){
		if(![rset containsObject:[w string]]){
			[rset addObject:[w string]];
			[resx addObject:w];
		}
	}
	return resx;
}

-(void) addWord:(FWord*) word
{
	[currentDict addWord:word];
}

-(void) addWords:(NSArray*) ws
{
	if(tmpBH){
		[bhDict addWords:ws];
		return;
	}

	if(currentInputMode==FWBXInputMode && wbxPYMix && tmpPY){
		[pyDict addWords:ws];
		return;
	}
	[currentDict addWords:ws];
}

-(void) removeWord:(FWord *)word
{
	if(tmpBH){
		[bhDict removeWord:word];
		return;
	}
	if(currentInputMode==FWBXInputMode && wbxPYMix && tmpPY){
		[pyDict removeWord:word];
		return;	
	}
	[currentDict removeWord:word];
}

-(NSArray*) userWords
{
	return [currentDict userWords];
}

-(void) saveUserWords
{
	[pyDict saveUserWords];
	[bhDict saveUserWords];
	[wbxDict saveUserWords];
}

@end
