//
//  FInputServer.m
//  FunInputToy2.0
//
//  Created by Feng Huajun on 08-1-23.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "FInputServer.h"
#import "FWord.h"
#import "FConf.h"
#import "FWordFactory.h"
#import <signal.h>

FInputServer* _fserver;

void clean_up( int dummy ) {
	[_fserver release];
	exit(0);
}

FWord* stringToWord(NSString* str){
	NSArray* a = [str componentsSeparatedByString:@","];
	if([a count]!=6) {
		NSLog(@"error at string to word (%@)",str);
		return nil;
	}
	FWord* w = [[[FWord alloc] init] autorelease];
	[w setCode:[a objectAtIndex:0]];
	[w setString:[a objectAtIndex:1]];
	[w setHit:[[a objectAtIndex:2] intValue]];
	[w setFreq:[[a objectAtIndex:3] intValue]];
	[w setNewFlag:[[a objectAtIndex:4] intValue]];
	[w setWordId:[[a objectAtIndex:5] intValue]];
	return w;
}

//This function use FWordFactory to generate a new word. When FWordFactory
//can't generate one, this will use FWord to generate a new
// word defaultly.
//
// 2008.6.28 added by Loya Liu <loya.liu@gmail.com>
//
FWord* stringToWordWithType(NSString* str,FInputEngine* engine){
	NSArray* a = [str componentsSeparatedByString:@","];
	if([a count]!=6) {
		NSLog(@"error at string to word with type (%@)",str);
		return nil;
	}
	FWord* w=nil;
	if([engine currentInputMode]==FPYInputMode){
		w=[FWordFactory genNewWord:FPY];
	}else if([engine currentInputMode]==FPYInputMode){
		w=[FWordFactory genNewWord:FWBX];
	}else w = [[[FWord alloc] init] autorelease];
	[w setCode:[a objectAtIndex:0]];
	[w setString:[a objectAtIndex:1]];
	[w setHit:[[a objectAtIndex:2] intValue]];
	[w setFreq:[[a objectAtIndex:3] intValue]];
	[w setNewFlag:[[a objectAtIndex:4] intValue]];
	[w setWordId:[[a objectAtIndex:5] intValue]];
	return w;
}

@implementation FInputServer

-(id) initWithPath:(NSString*) path
{
	self = [super init];
	if (self != nil) {
			_fserver = self;
			#ifndef IPHONE
			NSString* userPath = [NSHomeDirectory() stringByAppendingPathComponent:@".fit"];
			#else
			NSString* userPath = [NSHomeDirectory() stringByAppendingPathComponent:@"Library/FIT"];			
			#endif
			NSFileManager* fs = [NSFileManager defaultManager];
			if(![fs fileExistsAtPath:userPath]){
					[fs createDirectoryAtPath:userPath attributes:nil];
			}			

			NSString* conf = [userPath stringByAppendingString:@"/fitd.conf"];

			NSDictionary* setting = [FConf loadSettingFromFile:conf];
			
			NSString* dataPath;
			
			no_code_set = [NSCharacterSet characterSetWithCharactersInString:
					@"abcdefghijklmnopqrstuvwxyz'12345"];
			no_code_set = [[no_code_set invertedSet] retain];
			
			#ifndef IPHONE
			
			dataPath = @"/usr/share/fit";
			
			#endif
			
			#ifdef IPHONE
			
			dataPath = userPath;
			
			#endif
			
			//NSLog(@"dataPath=%@, userPath=%@, sock=%@",dataPath, userPath, path);
			engine = [[FInputEngine alloc] initWithDataPath:dataPath withUserPath:userPath withSetting:setting];
			//NSLog(@"engine init OK");
			signal( SIGTERM, clean_up );
			signal( SIGINT, clean_up );
			signal( SIGQUIT, clean_up );
			signal( SIGHUP, clean_up );
			/* 
			socket = [[AGSocket alloc] initWithFamily:PF_UNIX type:SOCK_STREAM protocol:0];
			AGSocketAddress* addr = [AGUnixSocketAddress addressWithPath:@"/tmp/fit.sock"];
			[socket setDelegate:self];	
			[socket listenOnAddress:addr];
			*/
			
			int len;
			struct sockaddr_un local;

			if ((sock = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
				fprintf(stderr,"socket init error %d (%s)\n", errno, strerror(errno));
				exit(1);
			}

			local.sun_family = AF_UNIX;
			strcpy(local.sun_path, [path fileSystemRepresentation]);
			//printf("listen to %s",local.sun_path);
			socket_path = (char *)[path fileSystemRepresentation];
			//printf("path = %s\n",local.sun_path);
			unlink(local.sun_path);
        
			len = strlen(local.sun_path) + 1 + sizeof(local.sun_family);
			
			if (bind(sock, (struct sockaddr *)&local, len) == -1) {
				fprintf(stderr,"socket bind error %d (%s)\n", errno, strerror(errno));
				exit(1);
			}

			if (listen(sock, 5) == -1) {
				fprintf(stderr,"socket bind error %d (%s)\n", errno, strerror(errno));
				exit(1);
			}
			
			remainWordsDict = [[NSMutableDictionary alloc] init];

	}
	return self;
}

-(void) start
{
		int s2, t;
		struct sockaddr_un remote;
		char str[1001];
		
		
        for(;;) {
            int n;
            t = sizeof(remote);
            if ((s2 = accept(sock, (struct sockaddr *)&remote, (socklen_t *)&t)) == -1) {
                fprintf(stderr,"socket accept error %d (%s)\n", errno, strerror(errno));
                exit(1);
            }
			
			NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
			
			n = recv(s2, str, 1000, 0);
			str[n] = 0;
			NSMutableString* receiveString = [NSString stringWithUTF8String:str];
            			
			NSString* res = [self handle:receiveString];
			
			const char* r = [res UTF8String];
			
			send(s2, r, strlen(r), 0);
			
			[pool release];

            close(s2);
        }


			

}


- (NSString *)handle:(NSString *)receiveString
{
	

	//NSLog(@"received %@",receiveString);
	
	NSArray* args = [[receiveString stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@" "]]
						componentsSeparatedByString:@" "];
	NSString* cmd = [args objectAtIndex:0];
	
	NSMutableString* result = [NSMutableString string];
	
	if([cmd isEqualToString:@"fix"]  && [args count] > 1){
		NSString* str = [args objectAtIndex:1];
		[result setString:[engine fixString:str]];
	}else if([cmd isEqualToString:@"search"]  && [args count] > 1){
		NSString* str = [args objectAtIndex:1];
		[remainWordsDict removeObjectForKey:str];
		NSArray* res = [engine wordsSearchByFixedString:str];
		NSEnumerator* e = [res objectEnumerator];
		FWord* w;
		int count = 0;
		while(w = [e nextObject]){
			if(count > 0) [result appendString:@" "];
			[result appendFormat:@"%@,%@,%d,%d,%d,%d",[w code],[w string],[w hit],[w freq],[w isNew], [w wordId]];
			count++;
			#ifdef IPHONE
			if(count > 11 && [[w string] length]==1){
				[remainWordsDict setObject:[res subarrayWithRange:NSMakeRange(count, [res count] - count)] forKey:str];
				break;
			}
			#endif
		}
	} if([cmd isEqualToString:@"more"]  && [args count] > 1){
		NSString* str = [args objectAtIndex:1];
		NSMutableArray* res = [remainWordsDict objectForKey:str];
		NSEnumerator* e = [res objectEnumerator];
		FWord* w;
		int count = 0;
		BOOL done = YES;
		while(w = [e nextObject]){
			if(count > 0) [result appendString:@" "];
			[result appendFormat:@"%@,%@,%d,%d,%d,%d",[w code],[w string],[w hit],[w freq],[w isNew], [w wordId]];
			count++;
			if(count > 10 ){
				[remainWordsDict removeObjectForKey:str];
				[remainWordsDict setObject:[res subarrayWithRange:NSMakeRange(count, [res count] - count)] forKey:str];
				done = NO;
				break;
			}
		}
		if(done) [remainWordsDict removeObjectForKey:str];
		//[res removeObjectsInRange:NSMakeRange(0, count)];
		//NSLog(@"rd = %@", remainWordsDict);
		
	}else if([cmd isEqualToString:@"related"]  && [args count] > 1){
		NSString* str = [args objectAtIndex:1];
		NSArray* res = [engine relatedWordsByString:str];
		NSEnumerator* e = [res objectEnumerator];
		FWord* w;
		int count = 0;
		while(w = [e nextObject]){
			if(count > 0) [result appendString:@" "];
			[result appendFormat:@"%@,%@,%d,%d,%d,%d",[w code],[w string],[w hit],[w freq],[w isNew], [w wordId]];
			count++;
			#ifdef IPHONE
			if(count > 200) break;
			#endif
		}
		#ifdef IPHONE
		/*
		if(count < 100){
			NSArray* resx = [engine relatedWordsByString:[NSString stringWithUTF8String:"Ａ"]];
			e = [resx objectEnumerator];
			while(w = [e nextObject]){
				if(count > 0) [result appendString:@" "];
				[result appendFormat:@"%@,%@,%d,%d,%d,%d",[w code],[w string],[w hit],[w freq],[w isNew], [w wordId]];
				count++;
				if(count > 100) break;
			}
		}
		*/		
		#endif
	}else if([cmd isEqualToString:@"add"]   && [args count] > 1){
		int i;
		NSMutableArray* ws = [NSMutableArray array];
		for(i=1;i<[args count];i++){
			FWord* w = stringToWordWithType([args objectAtIndex:i],engine);
			if(w&& [w code] && [[w code] length]){
				NSRange range = [[w code] rangeOfCharacterFromSet:no_code_set];
				//NSLog(@"range %@, %@ (%d,%d) %d", [w code], no_code_set, range.location, range.length, NSNotFound);
				if(range.location!=NSNotFound) continue;
				[ws addObject:w];
			}
		}
		if([ws count]) [engine addWords:ws];
	}else if([cmd isEqualToString:@"remove"]  && [args count] > 1){
		FWord* w = stringToWordWithType([args objectAtIndex:1],engine);
		if(w){
			[engine removeWord:w];
		}
	}else if([cmd isEqualToString:@"reload"]){
		#ifndef IPHONE
		NSString* userPath = [NSHomeDirectory() stringByAppendingPathComponent:@".fit"];
		#else
		NSString* userPath = [NSHomeDirectory() stringByAppendingPathComponent:@"Library/FIT"];			
		#endif
		NSString* conf = [userPath stringByAppendingString:@"/fitd.conf"];
		NSDictionary* setting = [FConf loadSettingFromFile:conf];
		[engine reloadSetting:setting];
	}else if([cmd isEqualToString:@"save"]){
		[remainWordsDict removeAllObjects];
		[engine saveUserWords];
	}else if([cmd isEqualToString:@"exit"]){
		clean_up(0);
	}
	
	//NSLog(@"sent %@", result);
	
	return result;
}

- (void) dealloc
{
	[remainWordsDict release];
	[no_code_set release];
	[engine release];
	close(sock);
	unlink(socket_path);
	[super dealloc];
}



@end
