//
//  FPYDictionary.m
//  FunInputToy2.0
//
//  Created by Feng Huajun on 08-1-7.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "FPYDictionary.h"
#import "FPYUtils.h"


@implementation FPYDictionary

- (id) initWithIndexFile:(NSString*) idxFile listFile:(NSString*) datFile dbFile:(NSString*) dbFile spSchemeFile: (NSString*) spFile
{
	self = [super init];
	if (self != nil) {
		tree = [[FTree alloc] initFromFile:[idxFile fileSystemRepresentation]];
		list = [[FList alloc] initFromFile:[datFile fileSystemRepresentation] withType:FPY];
		dtree = [[FDynamicTree alloc] initFromFile:[dbFile fileSystemRepresentation] withType:FPY];
		[FPYUtils loadSPSchemesFromFile:spFile];
	}
	return self;
}

- (FTree*) tree
{
	return tree;
}

- (FList*) list
{
	return list;
}

- (FDynamicTree*) dtree
{
	return dtree;
}

-(NSString *) fixInputString:(NSString *) str
{
	return [FPYUtils fixInputString:str];
}

-(NSArray *) searchWordsByFixedString:(NSString *) fixedString
{
	UInt32 p[1000];
	UInt32 len = 0;
	NSString* str = [fixedString stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"' "]];
	len = [FPYUtils getMutiCodeIndexs:p byPYString:str];
	if(!len) return nil;
	FWord* firstWord = nil;
	NSArray* candidateWords = [FPYUtils searchWordsByMutiCodeIndexs:p withTree:tree withList:list withDynamicTree:dtree];
	//NSLog(@"cws = %@",candidateWords);
	NSMutableArray* res = [NSMutableArray arrayWithArray:candidateWords];
	
	[res sortUsingSelector:@selector(compare:)];
	//NSLog(@"cws = %@,%d,%d",res,len,[[firstWord string] length]);
	
	if([res count]){
		firstWord = [res objectAtIndex:0];
	}
	if(len!=[[firstWord string] length]){
	  firstWord = [FPYUtils guestFirstWordWithMutiCodeIndexs:p withFirstWord:firstWord withTree:tree withList:list withDynamicTree:dtree];
	  [res insertObject:firstWord atIndex:0];
	  //NSLog(@"insert");
	}
	
	return res;
}

-(void) addWord:(FWord*) word
{
	[dtree addWord:word];
}

-(void) addWords:(NSArray*) ws
{
	#ifdef LEOPARD
	FWord* cw = nil;
	int i;
	for(i=0;i<[ws count];i++){
		FWord* w = [ws objectAtIndex:i];
		[self addWord:w];
		if(!cw) cw = w;
		else cw = [cw wordCombineWith:w];
	}
	if(i>1){
		[self addWord:cw];
	}
	#else
	int i;
	for(i=0;i<[ws count];i++){
		FWord* w = [ws objectAtIndex:i];
		[self addWord:w];
	}
	#endif
}

-(void) removeWord:(FWord *)word
{
	[dtree removeWord:word];
}

-(NSArray*) userWords
{
	return [dtree userWords];
}

-(NSArray*) userWordsForPost
{
	return [dtree userWordsForPost];
}

-(void) userWordsPostDone
{
	[dtree userWordsPostDone];
}


-(void) saveUserWords
{
	[dtree updateDBNow:nil];
}

- (void) dealloc
{
	[tree release];
	[list release];
	[dtree release];
	[super dealloc];
}



@end
