//
//  SQLiteDB.m
//  fit_mbcreater
//
//  Created by Feng Huajun on 07-9-19.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "FSQLiteDB.h"
#import <stdlib.h>

@implementation FSQLiteDB



-(id) initWithPath: (NSString*)path
{
	self = [super init];
	if (self != nil) {
		int err = sqlite3_open([path fileSystemRepresentation], &db);
		if (err) {
			NSLog(@"can not open database:%@",path);
			exit(1);
		}
	}
	return self;
}

-(NSString* ) md5:(NSString *) str
{
	#ifdef IPHONE
	return str;
	#else
	NSData *toHash = [str dataUsingEncoding:NSUTF8StringEncoding];
	unsigned char* digest = MD5([toHash bytes], [toHash length], NULL);
	NSString* s = [NSString stringWithFormat: @"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
			digest[0], digest[1], 
			digest[2], digest[3],
			digest[4], digest[5],
			digest[6], digest[7],
			digest[8], digest[9],
			digest[10], digest[11],
			digest[12], digest[13],
			digest[14], digest[15]];
	return s;
	#endif
}

-(void) enableQueryCache
{
	if(!queryCache){
		queryCacheRowNum = 1000;
		queryCache = [[NSMutableDictionary alloc] init];
	}
}

-(NSArray*) query: (NSString *)sql
{
	NSString* key = nil;
	if(queryCache){
		key = [self md5:sql];
		NSArray* cacheRows = [queryCache objectForKey:key];
		if(cacheRows) {
			//NSLog(@"cache hitted");
			return cacheRows;
		}
	}
	//NSLog(@"query: %@",sql);
	char* errMsg;
	char **result;
	int nrow,ncol;
	int err = sqlite3_get_table(db,[sql UTF8String],&result,&nrow,&ncol,&errMsg);
	if( err != SQLITE_OK ){
		NSLog(@"query sql error! sql:%@ message:%s", sql, errMsg);
		exit(1);		
	}
	
	NSMutableArray* rows = [[NSMutableArray alloc] initWithCapacity:nrow];
	[rows autorelease];


		
	if(nrow != 0) {
	
		NSMutableArray* colNames = [[NSMutableArray alloc] initWithCapacity: ncol];
	
		int idx = 0;
		int i,j;
	
		for(i = 0; i < ncol; i++){
			NSString* s = [[NSString alloc] initWithUTF8String:result[i]];
			[colNames addObject:s];
			[s release];
		}
	
		for(i = 0; i < nrow; i++ ) {
			NSMutableDictionary* cols = [[NSMutableDictionary alloc] initWithCapacity: ncol];
			for(j=0;j<ncol;j++){
				idx = (i+1)*ncol + j;
				//printf("s=%s\n",result[idx]);
				if(result[idx]){
					NSString* s = [[NSString alloc] initWithUTF8String:result[idx]];
					[cols setObject:s forKey:[colNames objectAtIndex:j]];
					[s release];
				}else{
					[cols setObject:@"" forKey:[colNames objectAtIndex:j]];
				}
			}
			[rows addObject:cols];
			[cols release];
		}
		[colNames release];
	}
	
	sqlite3_free_table(result);
	
	if(queryCache){
	   if([queryCache count] > queryCacheRowNum){
		[queryCache release];
		queryCache = [[NSMutableDictionary alloc] init];
	   }
	   [queryCache setObject:rows forKey:key];
	}
	
	return rows;
}

-(void) execute: (NSString *)sql
{
	char* errMsg;
	int err = sqlite3_exec(db,[sql UTF8String], NULL, 0, &errMsg);
	if(err!=SQLITE_OK){
        NSLog(@"excute sql error! sql:%@ message:%s", sql, errMsg);
        exit(1);	
	}
}

-(int) last_insert_id
{
	return sqlite3_last_insert_rowid(db);
}

- (void) dealloc
{
	if(queryCache) [queryCache release];
	sqlite3_close(db);
	[super dealloc];
}

-(void) beginTrans
{
		[self execute:@"BEGIN TRANSACTION;"];
}

-(void) endTrans
{
		[self execute:@"COMMIT;"];
}



@end
