//
//  FTree.m
//  FunInputToy2.0
//
//  Created by Feng Huajun on 07-12-22.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "FTree.h"
#import <sys/stat.h>
#import <sys/mman.h>

#define MAX_ARRAY_NUM 100000

@implementation FTree

- (id) initFromFile:(const char*) filename
{
	self = [super init];
	if (self != nil) {
		fd = -1;
		struct stat statbuf;
		size_t length;
		int r;
		fd = open(filename,O_RDONLY);
		if(fd==-1){
			fprintf(stderr,"could not open %s: error %d (%s)\n", filename, errno, strerror(errno));
			exit(1);
		}
		r = fstat(fd,&statbuf);
		if(r==-1){
			fprintf(stderr,"fstat %s failed: error %d (%s)\n", filename, errno, strerror(errno));
			exit(1);
		}		
		length = statbuf.st_size;
		
		
		#ifndef IPHONE
		
		treeArray = mmap(NULL, length, PROT_READ, MAP_PRIVATE, fd, 0);
		if((caddr_t)treeArray== (caddr_t)-1){
			fprintf(stderr,"mmap %s failed: error %d (%s)\n", filename, errno, strerror(errno));
			exit(1);
		}
		
		#else
		NSError* error;
		data = [[NSData alloc] initWithContentsOfFile:[NSString stringWithUTF8String:filename] options:NSMappedRead error:&error];
		treeArray = [data bytes];
		length =  [data length];
		/*
		malloc(length);
		if(read(fd, treeArray, length) == (ssize_t)-1){
			fprintf(stderr,"reading %s failed: error %d (%s)\n", filename, errno, strerror(errno));
			exit(1);
		} 
		 */
		
		#endif
					
		count = length / sizeof(FTreeNode);
	}
	cacheDict = [[NSMutableDictionary alloc] init];
	
	FTreeNode* parent = &treeArray[0];
	UInt32 i;
	i = parent->firstChild;
	while(i!=0){
		NSNumber* key = [NSNumber numberWithInt:treeArray[i].codeIndex];
		NSNumber* value = [NSNumber numberWithInt:i];
		[cacheDict setObject:value forKey:key];
		i = treeArray[i].nextSibling;
	}
	//NSLog(@"cacheDict=%@",cacheDict);
	return self;
	
}

-(UInt32) searchByCodeIndex:(UInt32) c withParentIndex: (UInt32) parentIndex
{
	
	if(parentIndex==0){
		NSNumber* key = [NSNumber numberWithInt:c];
		NSNumber* value = [cacheDict objectForKey:key];
		if(value==nil) return -1;
		return [value intValue];
	}
	
	FTreeNode* parent = &treeArray[parentIndex];
	UInt32 i, r;
	i = parent->firstChild;
	r = -1;
	while(i!=0){
		if(treeArray[i].codeIndex == c) {
			r = i;
			break;
		}
		i = treeArray[i].nextSibling;
	}
	return r;
}

-(UInt32) wordIndexByCodeIndexs:(UInt32 *) cc withLength:(UInt32) len
{
	UInt32 i=0, index=0, lastIndex=-1;
	for(i=0;i<len;i++){
		index = [self searchByCodeIndex:cc[i] withParentIndex:index];
		if(index==-1) {
			return -1;
		}
		lastIndex = index;
	}
	return treeArray[lastIndex].wordIndex;
}


/* search by code index array, modify node indexs, and return how many nodes are found */

-(UInt32) searchByCodeIndexs:(UInt32*) cc withParentIndex: (UInt32) parentIndex resultIndexs:(UInt32*) indexs
{
	UInt32 i, l,j,ci;

	if(parentIndex==0){
		j=0;
		l=0;
		NSNumber* num; 
		while(cc[j]!=0){
			num = [cacheDict objectForKey:[NSNumber numberWithInt:cc[j]]];
			if(num){
				indexs[l] = [num intValue];
				l++;
			}
			j++;
		}
		return l;
	}

	FTreeNode* parent = &treeArray[parentIndex];
	
	i = parent->firstChild;
	l = 0;
	while(i!=0){
		ci = treeArray[i].codeIndex;
		j=0;
		while(cc[j]!=0){
			if(ci==cc[j]){
				indexs[l]=i;
				l++;
			}
			j++;
		}
		i = treeArray[i].nextSibling;
	}
	return l;
}

-(UInt32) searchByMutiCodeIndexs:(UInt32*) xcc result:(UInt32*) result returnLength:(UInt32*) oLen returnIndex:(UInt32*) oIndex;
{
	
	UInt32 r[MAX_ARRAY_NUM];
	int ir[100], ir_count =-1;
	//bzero(r,sizeof(UInt32)*MAX_ARRAY_NUM);
	UInt32 i = 0, len,wi=1,ri=0,parentIndex=0;
	UInt32 wlen = 0, oi;
	r[0]=0;
	r[1]=0;
	
	//printf("hello\n");
	
	while(xcc[i]!=0){
		
		ir_count++;
		ir[ir_count] = i;
		//printf("ir[%d]=%d\n",ir_count,i);
		
		
		wi++;
		parentIndex = r[ri];
		UInt32 sum = 0;
		do{
			len = [self searchByCodeIndexs:&xcc[i] withParentIndex:parentIndex resultIndexs:&r[wi]];
			//printf("lllen = %d\n",len);
			wi+=len;
			r[wi] = 0;
			sum += len;
			ri++;
			parentIndex=r[ri];
		}while(parentIndex!=0);	
		//wi++;
		//r[wi] = 0;
		
		if(sum==0){
			oi = ir[ir_count];
			ir_count--;
			//printf("sum=%d, break;\n",sum);
			break;
		}
		r[wi] = 0;
		

		
		while(xcc[i]!=0)i++;
		i++;		
		ri++;
		wlen++;
		oi = i;
		//printf("normal\n",sum);
	}
	//r[wi]==0;
	//for(i=0;i<wi;i++) printf("r[%d]=%d\n",i,r[i]);
	//ir_count--;
	

	
	i=0;
	count=0;
	//NSLog(@"i=%d, ri=%d, wi=%d",i,ri,wi);
	int plen;
	
	do{
		//printf("oi=%d",oi);
	  while(r[wi]==0)wi--;
	  while(r[wi]!=0){
			//printf("treeArray[r[%d]].wordIndex=%d\n",oi,wi,treeArray[r[wi]].wordIndex);
			if(treeArray[r[wi]].wordIndex!=-1){
				UInt32 path[100],p;
				plen = [self getPathIndexs:path byIndex:r[wi]];
			//printf("plen=%d\n",plen);
				for(p=0;p<plen;p++) {
					result[i] = treeArray[path[p]].codeIndex;
				//printf("result[%d]=%d\n",i,result[i]);
					i++;
				}
				result[i] = treeArray[r[wi]].wordIndex;
			//printf("result[%d]=%d\n",i,result[i]);
			//printf("r[%d]=%d,index%d=%d\n",wi,r[wi],i,result[i]);			
				i++;
				count++;
			}
			wi--;
		}
		if(count!=0) break;
		//printf("shit\n");
		oi=ir[ir_count];
		ir_count--;
		//oi++;
	}while(YES);
	*oIndex = oi;
	*oLen = plen;
		
	return count;
}

-(UInt32) getPathIndexs:(UInt32*) indexs byIndex:(UInt32) index
{
	FTreeNode* node;
	int i = 0;
	UInt32 len, ii=index;
	UInt32 is[100];
	while(ii!=0){
			node = &treeArray[ii];
			is[i] = ii;
			i++;
			ii = node->parent;
	}
	len = i;
	//revert here;
	i--;
	int j=0;
	while(i>=0){
		indexs[j] = is[i];
		j++;
		i--;
	}
	/*
	float h = len/2.0-1;
	for(i=0;i<=h;i++){
		tmp = indexs[i];
		indexs[i] =  indexs[len-1-i];
		indexs[len-1-i] = tmp;
	}*/
	
	return len;
}

-(UInt32) count
{
	return count;
}


- (void) dealloc
{
	if(fd!=-1){
		
		#ifndef IPHONE
		munmap(treeArray, count*sizeof(FTreeNode));
		#endif
		
		close(fd);
	}
	if(!data) [data release];
	[super dealloc];
}

-(NSString*) description
{
	NSMutableString* ms = [NSMutableString string];
	int i;
	for(i=0;i<count;i++){
		[ms appendFormat:@"\n%d => (code = %d, index = %d, firstChild = %d, nextSibling = %d)", i,
			treeArray[i].codeIndex, treeArray[i].wordIndex, treeArray[i].firstChild, treeArray[i].nextSibling];
	}
	return ms;
}

@end
